#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators
{
	public class GapMomentum : Indicator
	{
		#region Variables
		private double				gap, upGaps, dnGaps;
		#endregion
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description			= @"The GapMomentum indicator as published in the January 2024 Technical Analysis of Stocks and Commodities article 'Gap Momentum' by Perry J. Kaufman.";
				Name				= "GapMomentum";
				IsOverlay			= false;
				Period				= 40;
				SignalPeriod		= 20;
				AddPlot(Brushes.Red, "GapRatio");
				AddPlot(Brushes.Blue, "Signal");
			}
		}

		protected override void OnBarUpdate()
		{
			if (CurrentBar < Period)
				return;

			for (int ix = 0; ix < Period; ix++)
			{
				gap = Open[ix] - Close[ix + 1];

				if (gap > 0)
				{
					upGaps += gap;
				}
				else if (gap < 0)
				{
					dnGaps -= gap;
				}
			}
			
			if (dnGaps == 0)
				GapRatio[0] = 1;
			else
				GapRatio[0] = 100 * upGaps / dnGaps;

			Signal[0] = SMA(GapRatio, SignalPeriod)[0];
		}

		#region Properties
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="Period", Description="Period", Order=1, GroupName="Parameters")]
		public int Period
		{ get; set; }

		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="SignalPeriod", Description="Signal line period", Order=2, GroupName="Parameters")]
		public int SignalPeriod
		{ get; set; }

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> GapRatio
		{
			get { return Values[0]; }
		}
		
		[Browsable(false)]
		[XmlIgnore]
		public Series<double> Signal
		{
			get { return Values[1]; }
		}
		#endregion

	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private GapMomentum[] cacheGapMomentum;
		public GapMomentum GapMomentum(int period, int signalPeriod)
		{
			return GapMomentum(Input, period, signalPeriod);
		}

		public GapMomentum GapMomentum(ISeries<double> input, int period, int signalPeriod)
		{
			if (cacheGapMomentum != null)
				for (int idx = 0; idx < cacheGapMomentum.Length; idx++)
					if (cacheGapMomentum[idx] != null && cacheGapMomentum[idx].Period == period && cacheGapMomentum[idx].SignalPeriod == signalPeriod && cacheGapMomentum[idx].EqualsInput(input))
						return cacheGapMomentum[idx];
			return CacheIndicator<GapMomentum>(new GapMomentum(){ Period = period, SignalPeriod = signalPeriod }, input, ref cacheGapMomentum);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.GapMomentum GapMomentum(int period, int signalPeriod)
		{
			return indicator.GapMomentum(Input, period, signalPeriod);
		}

		public Indicators.GapMomentum GapMomentum(ISeries<double> input , int period, int signalPeriod)
		{
			return indicator.GapMomentum(input, period, signalPeriod);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.GapMomentum GapMomentum(int period, int signalPeriod)
		{
			return indicator.GapMomentum(Input, period, signalPeriod);
		}

		public Indicators.GapMomentum GapMomentum(ISeries<double> input , int period, int signalPeriod)
		{
			return indicator.GapMomentum(input, period, signalPeriod);
		}
	}
}

#endregion
