#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.Indicators;
using NinjaTrader.NinjaScript.DrawingTools;
using System.IO;
#endregion

//This namespace holds Strategies in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Strategies
{
	public class GapMomentumStrategy : Strategy
	{
		#region Variables
		private int				size;
		private GapMomentum		gapMomentum;
		#endregion
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description			= @"The GapMomentum strategy as published in the January 2024 Technical Analysis of Stocks and Commodities article 'Gap Momentum' by Perry J. Kaufman.";
				Name				= "GapMomentumStrategy";
				Investment			= 100000;
				LongOnly			= false;
				Period 				= 40;
				SignalPeriod		= 20;
			}
			else if (State == State.DataLoaded)
			{
				gapMomentum = GapMomentum(Period, SignalPeriod);
				AddChartIndicator(gapMomentum);
			}
		}

		protected override void OnBarUpdate()
		{
			if (CurrentBar < BarsRequiredToTrade || !IsFirstTickOfBar)
				return;
			
			size = Convert.ToInt32(Investment / Close[0]);
			
			if (Position.MarketPosition != MarketPosition.Long && gapMomentum.Signal[0] > gapMomentum.Signal[1])
			{
				EnterLong(size, "Buy");
			}
			else if (Position.MarketPosition != MarketPosition.Short && gapMomentum.Signal[0] < gapMomentum.Signal[1])
			{
				if (LongOnly)
					ExitLong("ExitLong", "Buy");				
				else
					EnterShort(size, "Sell");
			}
		}

		#region Properties
		
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="Investment", Description="Investment amount in currency", Order=1, GroupName="Parameters")]
		public int Investment
		{ get; set; }
		
		[NinjaScriptProperty]
		[Display(Name="LongOnly", Description="Submit long trades only", Order=2, GroupName="Parameters")]
		public bool LongOnly
		{ get; set; }
		
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="Period", Description="Period of Gap Momentum indicator", Order=3, GroupName="Parameters")]
		public int Period
		{ get; set; }

		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="SignalPeriod", Description="Signal period of Gap Momentum indicator", Order=4, GroupName="Parameters")]
		public int SignalPeriod
		{ get; set; }
		#endregion

	}
}
